#include <stdio.h>
#include <string.h>

/* plaunch.c - parallel application launcher 
From OS X's Unix command line, plaunch uses Pooch to launch parallel jobs
Compile using: sudo cc plaunch.c -o /bin/plaunch

Copyright  2001-2  Dean E. Dauger, Dauger Research, http://daugerresearch.com/

This code requires Pooch version 1.1 or later to be installed on the local 
network of Macs.  It translates the parameters entered on the Unix command 
line into AppleScripts that communicate with Pooch.  A demonstration version 
of Pooch is available at: http://daugerresearch.com

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or accessing http://www.fsf.org/copyleft/gpl.html

****************************************************************

Version History:

2002 May 21 - Added -W, -, and --help options and changed the makefile 

2001 October 3 - Creation using cc on Mac OS X 10.1 

Thanks goes to the following for advice and recommendations:
  Daniel Gordon (NRL), Mark Chavira (UCLA EE/CS), 
  Charles Norton (JPL), Viktor Decyk (UCLA Physics)
*/


void printUsage(); 
void printUsage()
	{
	printf("\n"); 
	printf("Usage: \n\nplaunch executable [file1 file2 ...] -n addr1 addr2 ...\n"); 
	printf("plaunch executable [file1 file2 ...] -n nodeCount\t! searches for \"best\" nodes\n");
	printf("plaunch -j jobFile");
	printf("\t\t! \"plaunch -j\" prints a sample jobFile\n");
	printf("\nadditional options: \n"); 
	printf("\t -s : save the AppleScript file into a file named tempScript\n"); 
	printf("\t -m : not me! Don't include me in the node list by default\n"); 
	printf("\t -d : select a directory for the files to be on the remote nodes\n"); 
	printf("\t -K : skip the check for nodes being BUSY and launch anyway\n"); 
	printf("\t -Q : in case of a launch failure, Pooch queues the job for later relaunch\n"); 
	printf("\t -W : wait for user to select additional nodes or files and launch the job manually\n"); 
	printf("\t - or --help : print usage information\n"); 
	printf("\n"); 
	}

void generateJobFileSample(); 
void generateJobFileSample()
	{
	printf("\n"); 
	printf("executable\nfile1\nfile2\n...\n-n\naddr1\naddr2\n...\n-options\n"); 
	printf("\n\tor, using the \"best\" criteria, ...\n\n");
	printf("executable\nfile1\nfile2\n...\n-n \nnodeCount\n-options\n"); 
	printf("\n"); 
	}


#define MaxFiles	256
#define MaxNodes	256

int main(int argc, char *argv[])
	{int out=0; 

	if (argc<=1) printUsage(); 
	else {
		char *jobFileData=NULL; 
		int fileArgs[MaxFiles], numFileArgs=0;
		int nodeArgs[MaxNodes], numNodeArgs=0;
		int i, jobFileArg=0, subDirArg=0; 
		short proceed=1; 
		short jobFileFlag=0, readingNodes=0, saveScript=0, noSelfOption=0; 
		short skipBusyCheck=0, launchFailureQueuesJob=0, subdirectoryOption=0; 
		short waitForUserToLaunch=0; 

		for(i=1; i<argc; i++) {
			
			if ('-'==argv[i][0]) {/* an option */
				int j=1; 
				short keepScanning=1; 
				char c; 
				if (!(argv[i][1])) {
					printUsage(); 
					keepScanning=0; 
					proceed=0; 
					}
				while (keepScanning&&(c=argv[i][j++]))
					switch (c) {
						case 's': saveScript=1; break; 
						case 'm': noSelfOption=1; break; 
						case 'f': readingNodes=0; break; 
						case 'n': readingNodes=1; break; 
						case 'd': subdirectoryOption=1; break; 
						case 'K': skipBusyCheck=1; break; 
						case 'Q': launchFailureQueuesJob=1; break; 
						case '-': if ('h'!=argv[i][j]) break; //--help
						case 0: //-
						case 'h': //-help
						case '?': //-?
							printUsage(); keepScanning=0; proceed=0; break; 
						case 'W': waitForUserToLaunch=1; break; 
					        case 'j': /* only if first argument */
							if (1==i) if (!jobFileData) {
								
								if (argc<3) {
									proceed=0; 
									generateJobFileSample(); 
									}
								else {
									FILE *jfp=fopen(argv[2], "r"); 
									long length=0; 
									
									if (jfp) {
										fseek(jfp, 0, SEEK_END); 
										length=ftell(jfp); 
										if (length>0) {
											jobFileData=(char*)malloc(length); 
											
											if (jobFileData) {
												long argCount=0; 
												char *tp=jobFileData; 
												
												tp[0]=0; 
												
												rewind(jfp); /* read it all in */
												/* and parse it */
												while (EOF!=fscanf(jfp, "%s", tp)) {
													argCount++; 
													tp+=strlen(tp)+1; 
													}
												
												printf("parsing job file: %s has %ld arguments\n", argv[2], argCount); 
												
												argc=argCount+1; 
												argv=(char **)malloc(sizeof(char *)*(argc+1)); 
												argv[0]=tp=jobFileData; 
												for(i=1; i<argc; i++) {
													argv[i]=tp; 
													tp+=strlen(tp)+1; 
													}
												argv[argc]=NULL; 
												
												keepScanning=0; 
												jobFileFlag=1; 
												i=0; /*start over*/
												}
											
											}
										
										fclose(jfp); 
										}
									}
								}
							break; 
						default: printf("Unrecognized option: %c\n", c); 
							break; 
					}
				}
			else {/*
				if (jobFileFlag) {
					if (!jobFileArg) 
						jobFileArg=i; 
					}
				else */
				if (subdirectoryOption&&!subDirArg) {
					subDirArg=i; 
					}
				else 
				if (!readingNodes) {
					if (numFileArgs<MaxFiles) 
						fileArgs[numFileArgs++]=i; 
					}
				else {
					if (numNodeArgs<MaxNodes) 
						nodeArgs[numNodeArgs++]=i; 
					}
				
				
				}
			
			}
		
		if (proceed) 
			{
			if (numFileArgs<1) {
				printf("No files supplied.\n"); 
				proceed=0; 
				}
			
			if ((numNodeArgs<1)&&!waitForUserToLaunch) {
				printf("No node information supplied.\n"); 
				proceed=0; 
				}
				
			if (proceed) {//write out the AppleScript(s) and run them
				FILE *fp=fopen("tempScript", "w"); 
				if (fp) {
					rewind(fp); 
					fprintf(fp, "tell application \"Pooch App\"\n"); 
					fprintf(fp, " activate\n"); 
					fprintf(fp, " set j to make new job\n"); 
					fprintf(fp, "end tell\n"); 
					fclose(fp); 
					}
				out=system("osascript tempScript"); 
				
/* only because it seems to be highly inconvenient to convert POSIX paths to AppleScript file refs from here*/
				for(i=0; i<numFileArgs; i++) {
					char openstr[256]; 
					sprintf(openstr, "open -a \"Pooch App\" %s", argv[fileArgs[i]]); 
					out=system(openstr); 
					}
				
				fp=fopen("tempScript", "w"); 
				if (fp) {
					fprintf(fp, "tell application \"Pooch App\"\n"); 
					fprintf(fp, " set j to first job\n"); 
/*					for(i=0; i<numFileArgs; i++) {
						fprintf(fp, " make new file at j with data alias \"%s\"\n", argv[fileArgs[i]]); 
						}
*/					
					if (noSelfOption) 
						fprintf(fp, " delete the node list at j\n"); /*clear it*/
					
					if (numNodeArgs>0) 
						{int nodeCount=0; 
						sscanf(argv[nodeArgs[0]], "%d", &nodeCount); 
						i=strlen(argv[nodeArgs[0]]); 
						while (i--) {
							char c=argv[nodeArgs[0]][i]; 
							if ((c<'0')||(c>'9')) /* if any part of it is not a number */
								nodeCount=0; /* then it's an address */
							}
						if (nodeCount) {
							fprintf(fp, " make new node at j with properties (node scan for best %d)\n", nodeCount); 
							fprintf(fp, " delete node %d of j\n", nodeCount+1); 
							}
						else {
							for(i=0; i<numNodeArgs; i++) {
								fprintf(fp, " make new node at j with properties {name:\"%s\", address:\"%s\"}\n", argv[nodeArgs[i]], argv[nodeArgs[i]]); 
								}
							}
						}
					//job options generally added in this fashion
					if (skipBusyCheck) 
						fprintf(fp, " set BUSY check of j to no\n"); 
					if (launchFailureQueuesJob) 
						fprintf(fp, " set launch failure queues job of j to yes\n"); 
					if (subDirArg) 
						fprintf(fp, " set target subfolder of j to \"%s\"\n", argv[subDirArg]); 
					
					if (!waitForUserToLaunch) 
						fprintf(fp, " launch j\n"); 
					
					fprintf(fp, "end tell\n"); 
					
					out=fclose(fp); 
					}
				
				/*see the man pages on osacompile, osascript*/
				out=system("osascript tempScript"); 
				if (!saveScript) remove("tempScript"); 
				
				/* I think it's polite to clean up after myself */
				if (jobFileFlag) {
					free(jobFileData); 
					jobFileData=NULL; 
					free(argv); 
					}
				
				}
			
			}
		
    	}

	return out; 
	}
